"use strict";
/* eslint no-console: off */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var chalk_1 = tslib_1.__importDefault(require("chalk"));
var yargs_1 = tslib_1.__importDefault(require("yargs"));
var types_1 = require("./types");
var _1 = require(".");
var argv = yargs_1.default
    .usage('Usage: $0 [options]')
    .option('watch', {
    required: false,
    default: false,
    type: 'boolean',
    describe: 'Watch the GraphQL files for changes and re-run the generation',
})
    .option('cwd', {
    required: false,
    default: process.cwd(),
    normalize: true,
    type: 'string',
    describe: 'Working directory to use',
})
    .option('schema-types-path', {
    required: true,
    normalize: true,
    type: 'string',
    describe: 'The path to output concrete schema types',
})
    .option('add-typename', {
    required: false,
    default: true,
    type: 'boolean',
    describe: 'Add a __typename field to every object type',
})
    .option('export-format', {
    required: false,
    describe: 'The format to use for values exported from GraphQL documents',
    choices: [types_1.ExportFormat.Document, types_1.ExportFormat.Simple],
})
    .option('enum-format', {
    required: false,
    describe: 'The format in which to generate case names for enum types in the schema',
    choices: [
        types_1.EnumFormat.CamelCase,
        types_1.EnumFormat.PascalCase,
        types_1.EnumFormat.SnakeCase,
        types_1.EnumFormat.ScreamingSnakeCase,
    ],
})
    .option('custom-scalars', {
    required: false,
    default: '{}',
    type: 'string',
    describe: 'A JSON-serialized object where the key is the name of a custom scalar, and the value is an object with the name and package from which to import the type to use for that scalar',
})
    .help().argv;
var builder = new _1.Builder({
    cwd: argv.cwd,
    schemaTypesPath: argv['schema-types-path'],
    addTypename: argv['add-typename'],
    enumFormat: argv['enum-format'],
    customScalars: normalizeCustomScalars(argv['custom-scalars']),
});
function normalizeCustomScalars(customScalarOption) {
    try {
        var result = JSON.parse(customScalarOption);
        return typeof result === 'object' ? result : undefined;
    }
    catch (error) {
        return undefined;
    }
}
var schemas = [];
var docs = [];
var BUILT = chalk_1.default.inverse.bold.green(' BUILT ');
var ERROR = chalk_1.default.inverse.bold.red(' ERROR ');
builder.on('start:docs', function () {
    docs.length = 0;
    console.log();
});
builder.on('start:schema', function () {
    schemas.length = 0;
    console.log();
});
builder.on('build:docs', function (doc) {
    docs.push(doc);
});
builder.on('build:schema', function (schema) {
    schemas.push(schema);
});
builder.on('end:docs', function () {
    docs
        .sort(function (_a, _b) {
        var documentPathA = _a.documentPath;
        var documentPathB = _b.documentPath;
        return documentPathA.localeCompare(documentPathB);
    })
        .forEach(function (_a) {
        var documentPath = _a.documentPath, definitionPath = _a.definitionPath;
        console.log(BUILT + " " + chalk_1.default.dim(documentPath) + " \u2192 " + definitionPath);
    });
});
builder.on('end:schema', function () {
    schemas
        .sort(function (_a, _b) {
        var schemaPathA = _a.schemaPath;
        var schemaPathB = _b.schemaPath;
        return schemaPathA.localeCompare(schemaPathB);
    })
        .forEach(function (_a) {
        var schemaPath = _a.schemaPath, schemaTypesPath = _a.schemaTypesPath;
        console.log(BUILT + " " + chalk_1.default.dim(schemaPath) + " \u2192 " + schemaTypesPath);
    });
});
builder.on('error', function (error) {
    console.log(ERROR + " " + error.message);
    if (error.stack) {
        console.log(chalk_1.default.dim(error.stack));
    }
    console.log();
    if (!argv.watch) {
        process.exit(1);
    }
});
builder.run({ watch: argv.watch });
